@@ -94,7 +94,7 @@ class SelectionMixin(object):
 
         def _model_evaluator(allowed_models):
             def test(value):
-                return value[0] in allowed_models
+                return value[0] in allowed_models or not allowed_models
             return test
 
         if field.attrs['type'] == 'reference':
@@ -186,7 +186,7 @@ class PopdownMixin(object):
         model = entry.get_model()
         for i, values in enumerate(model):
             if values[1] == value:
-                entry.set_active(i)
+                gobject.idle_add(entry.set_active, i)
                 break
 
     def get_popdown_value(self, entry, index=1):
