@@ -1,7 +1,5 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
-
 import tryton.common as common
 
 
@@ -19,7 +17,7 @@ class NoModal(object):
 
     def register(self):
         from tryton.gui.main import Main
-        main = Main.get_main()
+        main = Main()
         self.page = main.get_page()
         if not self.page:
             self.page = main
@@ -27,10 +25,9 @@ class NoModal(object):
         self.sensible_widget.props.sensitive = False
 
     def destroy(self):
-        self.page.dialogs.remove(self)
-        # Test if the parent is not already destroyed
-        if self.parent not in gtk.window_list_toplevels():
+        if not self.page:
             return
+        self.page.dialogs.remove(self)
         self.parent.present()
         self.sensible_widget.props.sensitive = True
         for focus in self.parent_focus:
