@@ -3,7 +3,7 @@
 import calendar
 import datetime
 import goocalendar
-from dates_period import DatesPeriod
+from .dates_period import DatesPeriod
 
 
 class Calendar_(goocalendar.Calendar):
@@ -22,6 +22,8 @@ class Calendar_(goocalendar.Calendar):
         dtstart = self.attrs['dtstart']
         record[dtstart].set(record, datetime.datetime.combine(selected_date,
             datetime.time(0)))
+        record.on_change([dtstart])
+        record.on_change_with([dtstart])
 
     def get_displayed_period(self):
         cal = calendar.Calendar(self.firstweekday)
@@ -72,6 +74,9 @@ class Calendar_(goocalendar.Calendar):
         return text_color, bg_color
 
     def display(self, group):
+        def is_date_only(value):
+            return (isinstance(value, datetime.date)
+                and not isinstance(value, datetime.datetime))
         dtstart = self.attrs['dtstart']
         dtend = self.attrs.get('dtend')
         if self.screen.current_record:
@@ -96,14 +101,11 @@ class Calendar_(goocalendar.Calendar):
             else:
                 end = None
             midnight = datetime.time(0)
-            all_day = False
+            all_day = is_date_only(start) and (not end or is_date_only(end))
             if not isinstance(start, datetime.datetime):
                 start = datetime.datetime.combine(start, midnight)
             if end and not isinstance(end, datetime.datetime):
                 end = datetime.datetime.combine(end, midnight)
-                all_day = True
-            elif not end:
-                all_day = True
 
             # Skip invalid event
             if end is not None and start > end:
