@@ -41,6 +41,8 @@ class DictEntry(object):
         widget.connect('key-press-event', self.parent_widget.send_modified)
         widget.connect('focus-out-event',
             lambda w, e: self.parent_widget._focus_out())
+        widget.props.activates_default = True
+        widget.connect('activate', self.parent_widget.sig_activate)
         return widget
 
     def modified(self, value):
@@ -61,7 +63,7 @@ class DictBooleanEntry(DictEntry):
 
     def create_widget(self):
         widget = gtk.CheckButton()
-        widget.connect('toggled', self.parent_widget.send_modified)
+        widget.connect('toggled', self.parent_widget.sig_activate)
         widget.connect('focus-out-event', lambda w, e:
             self.parent_widget._focus_out())
         return widget
@@ -70,7 +72,12 @@ class DictBooleanEntry(DictEntry):
         return self.widget.props.active
 
     def set_value(self, value):
-        self.widget.props.active = bool(value)
+        self.widget.handler_block_by_func(self.parent_widget.sig_activate)
+        try:
+            self.widget.props.active = bool(value)
+        finally:
+            self.widget.handler_unblock_by_func(
+                self.parent_widget.sig_activate)
 
     def set_readonly(self, readonly):
         self.widget.set_sensitive(not readonly)
@@ -180,7 +187,7 @@ class DictIntegerEntry(DictEntry):
 
     def set_value(self, value):
         if value is not None:
-            txt_val = locale.format('%d', value, True)
+            txt_val = locale.format_string('%d', value, True)
         else:
             txt_val = ''
         self.widget.set_text(txt_val)
@@ -236,7 +243,8 @@ class DictFloatEntry(DictIntegerEntry):
     def set_value(self, value):
         digits = self.digits()
         if value is not None:
-            txt_val = locale.format('%.' + str(digits[1]) + 'f', value, True)
+            txt_val = locale.localize(
+                '{0:.{1}f}'.format(value, digits[1]), True)
         else:
             txt_val = ''
         self.widget.set_width_chars(sum(digits))
@@ -265,8 +273,10 @@ class DictDateTimeEntry(DictEntry):
         record = self.parent_widget.record
         field = self.parent_widget.field
         if record and field:
-            format_ = field.time_format(record)
-            widget.props.format = format_
+            date_format = field.date_format(record)
+            time_format = field.time_format(record)
+            widget.props.date_format = date_format
+            widget.props.time_format = time_format
         widget.connect('key_press_event', self.parent_widget.send_modified)
         widget.connect('focus-out-event', lambda w, e:
             self.parent_widget._focus_out())
@@ -278,6 +288,15 @@ class DictDateTimeEntry(DictEntry):
     def set_value(self, value):
         self.widget.props.value = timezoned_date(value)
 
+    def set_readonly(self, readonly):
+        for child in self.widget.get_children():
+            if isinstance(child, Gtk.Entry):
+                child.set_editable(not readonly)
+                child.set_icon_sensitive(
+                    Gtk.EntryIconPosition.PRIMARY, not readonly)
+            elif isinstance(child, Gtk.ComboBox):
+                child.set_sensitive(not readonly)
+
 
 class DictDateEntry(DictEntry):
     expand = False
@@ -301,6 +320,11 @@ class DictDateEntry(DictEntry):
     def set_value(self, value):
         self.widget.props.value = value
 
+    def set_readonly(self, readonly):
+        super().set_readonly(readonly)
+        self.widget.set_icon_sensitive(
+            Gtk.EntryIconPosition.PRIMARY, not readonly)
+
 
 DICT_ENTRIES = {
     'char': DictEntry,
@@ -472,10 +496,7 @@ class DictWidget(Widget):
         alignment.add(hbox)
         n_rows = self.table.props.n_rows
         self.table.resize(n_rows + 1, 3)
-        if gtk.widget_get_default_direction() == gtk.TEXT_DIR_RTL:
-            text = _(':') + key_schema['string']
-        else:
-            text = key_schema['string'] + _(':')
+        text = key_schema['string'] + _(':')
         label = gtk.Label(set_underline(text))
         label.set_use_underline(True)
         label.set_alignment(1., .5)
