@@ -1049,6 +1049,7 @@ class ViewTree(View):
             if not name:
                 continue
             widget = self.get_column_widget(column)
+            widget.set_editable()
             if decoder.decode(widget.attrs.get('tree_invisible', '0')):
                 column.set_visible(False)
             elif name == self.screen.exclude_field:
@@ -1105,14 +1106,15 @@ class ViewTree(View):
                     selected_sum = common.timedelta.format(
                         selected_sum, converter)
                     sum_ = common.timedelta.format(sum_, converter)
-                elif digit:
-                    selected_sum = locale.format(
-                        '%.*f', (digit, selected_sum or 0), True)
-                    sum_ = locale.format('%.*f', (digit, sum_ or 0), True)
+                elif digit is not None:
+                    selected_sum = locale.localize(
+                        '{0:.{1}f}'.format(selected_sum or 0, digit), True)
+                    sum_ = locale.localize(
+                        '{0:.{1}f}'.format(sum_ or 0, digit), True)
                 else:
-                    selected_sum = locale.format(
-                        '%s', selected_sum or 0, True)
-                    sum_ = locale.format('%s', sum_ or 0, True)
+                    selected_sum = locale.localize(
+                        '{}'.format(selected_sum or 0), True)
+                    sum_ = locale.localize('{}'.format(sum_ or 0), True)
 
                 text = '%s / %s' % (selected_sum, sum_)
             else:
