@@ -402,13 +402,17 @@ class WinForm(NoModal, InfoBar):
         if (self.screen.current_record
                 and not readonly
                 and response_id in cancel_responses):
+            record = self.screen.current_record
+            added = 'id' in record.modified_fields
             if (self.screen.current_record.id < 0
                     or self.save_current):
                 self.screen.cancel_current(self._initial_value)
-            elif self.screen.current_record.modified:
-                self.screen.current_record.cancel()
-                self.screen.current_record.reload()
-                self.screen.current_record.signal('record-changed')
+            elif record.modified:
+                record.cancel()
+                record.reload()
+                record.signal('record-changed')
+            if added:
+                record.modified_fields.setdefault('id')
             result = False
         else:
             result = response_id not in cancel_responses
