@@ -4,23 +4,7 @@ import json
 import datetime
 from decimal import Decimal
 from dateutil.relativedelta import relativedelta
-from functools import reduce, wraps
-
-
-def reduced_type(types):
-    types = types.copy()
-    for k, r in [(long, int), (str, basestring), (unicode, basestring)]:
-        if k in types:
-            types.remove(k)
-            types.add(r)
-    return types
-
-
-def reduce_type(func):
-    @wraps(func)
-    def wrapper(*args, **kwargs):
-        return reduced_type(func(*args, **kwargs))
-    return wrapper
+from functools import reduce
 
 
 class PYSON(object):
@@ -36,20 +20,20 @@ class PYSON(object):
         raise NotImplementedError
 
     def __invert__(self):
-        if self.types() != set([bool]):
+        if self.types() != {bool}:
             return Not(Bool(self))
         else:
             return Not(self)
 
     def __and__(self, other):
         if (isinstance(other, PYSON)
-                and other.types() != set([bool])):
+                and other.types() != {bool}):
             other = Bool(other)
         if (isinstance(self, And)
                 and not isinstance(self, Or)):
             self._statements.append(other)
             return self
-        if self.types() != set([bool]):
+        if self.types() != {bool}:
             return And(Bool(self), other)
         else:
             return And(self, other)
@@ -58,12 +42,12 @@ class PYSON(object):
 
     def __or__(self, other):
         if (isinstance(other, PYSON)
-                and other.types() != set([bool])):
+                and other.types() != {bool}):
             other = Bool(other)
         if isinstance(self, Or):
             self._statements.append(other)
             return self
-        if self.types() != set([bool]):
+        if self.types() != {bool}:
             return Or(Bool(self), other)
         else:
             return Or(self, other)
@@ -161,12 +145,11 @@ class Eval(PYSON):
             'd': self._default,
             }
 
-    @reduce_type
     def types(self):
         if isinstance(self._default, PYSON):
             return self._default.types()
         else:
-            return set([type(self._default)])
+            return {type(self._default)}
 
     @staticmethod
     def eval(dct, context):
@@ -178,9 +161,10 @@ class Not(PYSON):
     def __init__(self, v):
         super(Not, self).__init__()
         if isinstance(v, PYSON):
-            assert v.types() == set([bool]), 'value must be boolean'
-        else:
-            assert isinstance(v, bool), 'value must be boolean'
+            if v.types() != {bool}:
+                v = Bool(v)
+        elif not isinstance(v, bool):
+            v = bool(v)
         self._value = v
 
     @property
@@ -194,7 +178,7 @@ class Not(PYSON):
             }
 
     def types(self):
-        return set([bool])
+        return {bool}
 
     @staticmethod
     def eval(dct, context):
@@ -218,7 +202,7 @@ class Bool(PYSON):
             }
 
     def types(self):
-        return set([bool])
+        return {bool}
 
     @staticmethod
     def eval(dct, context):
@@ -230,13 +214,12 @@ class And(PYSON):
     def __init__(self, *statements, **kwargs):
         super(And, self).__init__()
         statements = list(statements) + kwargs.get('s', [])
-        for statement in statements:
+        for i, statement in enumerate(list(statements)):
             if isinstance(statement, PYSON):
-                assert statement.types() == set([bool]), \
-                    'statement must be boolean'
-            else:
-                assert isinstance(statement, bool), \
-                    'statement must be boolean'
+                if statement.types() != {bool}:
+                    statements[i] = Bool(statement)
+            elif not isinstance(statement, bool):
+                statements[i] = bool(statement)
         assert len(statements) >= 2, 'must have at least 2 statements'
         self._statements = statements
 
@@ -251,7 +234,7 @@ class And(PYSON):
             }
 
     def types(self):
-        return set([bool])
+        return {bool}
 
     @staticmethod
     def eval(dct, context):
@@ -278,11 +261,11 @@ class Equal(PYSON):
         if isinstance(statement1, PYSON):
             types1 = statement1.types()
         else:
-            types1 = reduced_type(set([type(s1)]))
+            types1 = {type(s1)}
         if isinstance(statement2, PYSON):
             types2 = statement2.types()
         else:
-            types2 = reduced_type(set([type(s2)]))
+            types2 = {type(s2)}
         assert types1 == types2, 'statements must have the same type'
         self._statement1 = statement1
         self._statement2 = statement2
@@ -299,7 +282,7 @@ class Equal(PYSON):
             }
 
     def types(self):
-        return set([bool])
+        return {bool}
 
     @staticmethod
     def eval(dct, context):
@@ -313,15 +296,16 @@ class Greater(PYSON):
         super(Greater, self).__init__()
         for i in (statement1, statement2):
             if isinstance(i, PYSON):
-                assert i.types().issubset({int, long, float, type(None)}), \
+                assert i.types().issubset({int, float, type(None)}), \
                     'statement must be an integer or a float'
             else:
-                assert isinstance(i, (int, long, float, type(None))), \
+                assert isinstance(i, (int, float, type(None))), \
                     'statement must be an integer or a float'
         if isinstance(equal, PYSON):
-            assert equal.types() == set([bool])
-        else:
-            assert isinstance(equal, bool)
+            if equal.types() != {bool}:
+                equal = Bool(equal)
+        elif not isinstance(equal, bool):
+            equal = bool(equal)
         self._statement1 = statement1
         self._statement2 = statement2
         self._equal = equal
@@ -339,14 +323,14 @@ class Greater(PYSON):
             }
 
     def types(self):
-        return set([bool])
+        return {bool}
 
     @staticmethod
     def _convert(dct):
         for i in ('s1', 's2'):
             if dct[i] is None:
                 dct[i] = 0.0
-            if not isinstance(dct[i], (int, long, float)):
+            if not isinstance(dct[i], (int, float)):
                 dct = dct.copy()
                 dct[i] = float(dct[i])
         return dct
@@ -382,18 +366,18 @@ class If(PYSON):
         condition, then_statement, else_statement = c, t, e
         super(If, self).__init__()
         if isinstance(condition, PYSON):
-            assert condition.types() == set([bool]), \
-                'condition must be boolean'
-        else:
-            assert isinstance(condition, bool), 'condition must be boolean'
+            if condition.types() != {bool}:
+                condition = Bool(condition)
+        elif not isinstance(condition, bool):
+            condition = bool(condition)
         if isinstance(then_statement, PYSON):
             then_types = then_statement.types()
         else:
-            then_types = reduced_type(set([type(then_statement)]))
+            then_types = {type(then_statement)}
         if isinstance(else_statement, PYSON):
             else_types = else_statement.types()
         else:
-            else_types = reduced_type(set([type(else_statement)]))
+            else_types = {type(else_statement)}
         assert then_types == else_types, \
             'then and else statements must be the same type'
         self._condition = condition
@@ -412,12 +396,11 @@ class If(PYSON):
             'e': self._else_statement,
             }
 
-    @reduce_type
     def types(self):
         if isinstance(self._then_statement, PYSON):
             return self._then_statement.types()
         else:
-            return set([type(self._then_statement)])
+            return {type(self._then_statement)}
 
     @staticmethod
     def eval(dct, context):
@@ -433,14 +416,14 @@ class Get(PYSON):
         obj, key, default = v, k, d
         super(Get, self).__init__()
         if isinstance(obj, PYSON):
-            assert obj.types() == set([dict]), 'obj must be a dict'
+            assert obj.types() == {dict}, 'obj must be a dict'
         else:
             assert isinstance(obj, dict), 'obj must be a dict'
         self._obj = obj
         if isinstance(key, PYSON):
-            assert key.types() == set([basestring]), 'key must be a string'
+            assert key.types() == {str}, 'key must be a string'
         else:
-            assert isinstance(key, basestring), 'key must be a string'
+            assert isinstance(key, str), 'key must be a string'
         self._key = key
         self._default = default
 
@@ -456,12 +439,11 @@ class Get(PYSON):
             'd': self._default,
             }
 
-    @reduce_type
     def types(self):
         if isinstance(self._default, PYSON):
             return self._default.types()
         else:
-            return set([type(self._default)])
+            return {type(self._default)}
 
     @staticmethod
     def eval(dct, context):
@@ -474,20 +456,20 @@ class In(PYSON):
         key, obj = k, v
         super(In, self).__init__()
         if isinstance(key, PYSON):
-            assert key.types().issubset(set([basestring, int])), \
+            assert key.types().issubset({str, int}), \
                 'key must be a string or an integer or a long'
         else:
-            assert isinstance(key, (basestring, int, long)), \
+            assert isinstance(key, (str, int)), \
                 'key must be a string or an integer or a long'
         if isinstance(obj, PYSON):
-            assert obj.types().issubset(set([dict, list])), \
+            assert obj.types().issubset({dict, list}), \
                 'obj must be a dict or a list'
-            if obj.types() == set([dict]):
-                assert isinstance(key, basestring), 'key must be a string'
+            if obj.types() == {dict}:
+                assert isinstance(key, str), 'key must be a string'
         else:
             assert isinstance(obj, (dict, list))
             if isinstance(obj, dict):
-                assert isinstance(key, basestring), 'key must be a string'
+                assert isinstance(key, str), 'key must be a string'
         self._key = key
         self._obj = obj
 
@@ -503,7 +485,7 @@ class In(PYSON):
             }
 
     def types(self):
-        return set([bool])
+        return {bool}
 
     @staticmethod
     def eval(dct, context):
@@ -523,10 +505,10 @@ class Date(PYSON):
         super(Date, self).__init__()
         for i in (year, month, day, delta_years, delta_months, delta_days):
             if isinstance(i, PYSON):
-                assert i.types().issubset(set([int, long, type(None)])), \
+                assert i.types().issubset({int, type(None)}), \
                     '%s must be an integer or None' % (i,)
             else:
-                assert isinstance(i, (int, long, type(None))), \
+                assert isinstance(i, (int, type(None))), \
                     '%s must be an integer or None' % (i,)
         self._year = year
         self._month = month
@@ -552,7 +534,7 @@ class Date(PYSON):
             }
 
     def types(self):
-        return set([datetime.date])
+        return {datetime.date}
 
     @staticmethod
     def eval(dct, context):
@@ -587,10 +569,10 @@ class DateTime(Date):
         for i in (hour, minute, second, microsecond,
                 delta_hours, delta_minutes, delta_seconds, delta_microseconds):
             if isinstance(i, PYSON):
-                assert i.types() == set([int, type(None)]), \
+                assert i.types() == {int, type(None)}, \
                     '%s must be an integer or None' % (i,)
             else:
-                assert isinstance(i, (int, long, type(None))), \
+                assert isinstance(i, (int, type(None))), \
                     '%s must be an integer or None' % (i,)
         self._hour = hour
         self._minute = minute
@@ -624,11 +606,11 @@ class DateTime(Date):
         return res
 
     def types(self):
-        return set([datetime.datetime])
+        return {datetime.datetime}
 
     @staticmethod
     def eval(dct, context):
-        return datetime.datetime.now() + relativedelta(
+        return datetime.datetime.utcnow() + relativedelta(
             year=dct['y'],
             month=dct['M'],
             day=dct['d'],
@@ -646,15 +628,53 @@ class DateTime(Date):
             )
 
 
+class TimeDelta(PYSON):
+
+    def __init__(self, days=0, seconds=0, microseconds=0):
+        for i in [days, seconds, microseconds]:
+            if isinstance(i, PYSON):
+                assert i.types().issubset({int, float}), \
+                        '%s must be an integer' % (i,)
+            else:
+                assert isinstance(i, (int, float)), \
+                        '%s must be an integer' % (i,)
+        self._days = days
+        self._seconds = seconds
+        self._microseconds = microseconds
+
+    @property
+    def __repr_params__(self):
+        return self._days, self._seconds, self._microseconds
+
+    def pyson(self):
+        return {
+            '__class__': 'TimeDelta',
+            'd': self._days,
+            's': self._seconds,
+            'm': self._microseconds,
+            }
+
+    def types(self):
+        return {datetime.timedelta}
+
+    @staticmethod
+    def eval(dct, context):
+        return datetime.timedelta(
+            days=dct['d'],
+            seconds=dct['s'],
+            microseconds=dct['m'],
+            )
+
+
 class Len(PYSON):
 
     def __init__(self, v):
         super(Len, self).__init__()
         if isinstance(v, PYSON):
-            assert v.types().issubset(set([dict, list, basestring])), \
+            assert v.types().issubset({dict, list, str}), \
                 'value must be a dict or a list or a string'
         else:
-            assert isinstance(v, (dict, list, basestring)), \
+            assert isinstance(v, (dict, list, str)), \
                 'value must be a dict or list or a string'
         self._value = v
 
@@ -669,12 +689,13 @@ class Len(PYSON):
             }
 
     def types(self):
-        return set([int])
+        return {int}
 
     @staticmethod
     def eval(dct, context):
         return len(dct['v'])
 
+
 CONTEXT = {
     'Eval': Eval,
     'Not': Not,
@@ -689,5 +710,6 @@ CONTEXT = {
     'In': In,
     'Date': Date,
     'DateTime': DateTime,
+    'TimeDelta': TimeDelta,
     'Len': Len,
-    }
+}
